"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const path_1 = require("path");
function readBuffer(fd, buffer, length) {
    return new Promise((r, j) => fs.read(fd, buffer, 0, length || buffer.length, null, (err, bytes) => err ? j(err) : r(bytes)));
}
const intBuffer = Buffer.alloc(6);
function readInt(fd) {
    return new Promise((r, j) => fs.read(fd, intBuffer, 0, 6, null, (err, bytes) => err ? j(err) : r(intBuffer.readIntBE(0, 6))));
}
function addParentPaths(name, index) {
    const parent = path_1.dirname(name);
    if (parent && !index[parent]) {
        index[parent] = {
            size: 0,
            isDirectory: () => true,
            isSymbolicLink: () => false,
            isBlockDevice: () => false,
            isCharacterDevice: () => false,
            isFile: () => false,
            isFIFO: () => false,
            isSocket: () => false,
            dev: 1,
            ino: 1,
            mode: 1,
            nlink: 1,
            uid: 1,
            gid: 1,
            rdev: 1,
            blksize: 1,
            blocks: 1,
            atimeMs: 1,
            mtimeMs: 1,
            ctimeMs: 1,
            birthtimeMs: 1,
            atime: new Date(),
            mtime: new Date(),
            ctime: new Date(),
            birthtime: new Date(),
        };
        return addParentPaths(parent, index);
    }
    return index;
}
function staticfs(sourcePath) {
    return __awaiter(this, void 0, void 0, function* () {
        // read the index first
        // 
        const index = {};
        const fd = fs.openSync(sourcePath, 'r');
        let buf = Buffer.alloc(1024 * 128);
        let dataOffset = yield readInt(fd);
        do {
            const nameSz = yield readInt(fd);
            if (nameSz == 0) {
                break;
            }
            const dataSz = yield readInt(fd);
            while (nameSz > buf.length) {
                buf = Buffer.alloc(buf.length * 2);
            }
            yield readBuffer(fd, buf, nameSz);
            const name = buf.toString('utf-8', 0, nameSz);
            // add entry for file into index
            index[name] = {
                ino: dataOffset,
                size: dataSz,
                isDirectory: () => false,
                isSymbolicLink: () => false,
                isBlockDevice: () => false,
                isCharacterDevice: () => false,
                isFile: () => true,
                isFIFO: () => false,
                isSocket: () => false,
            };
            // ensure parent path has a directory entry
            addParentPaths(name, index);
            dataOffset += dataSz;
        } while (true);
        return {
            readFileSync: (filepath, options) => {
                const item = index[filepath];
                if (item && item.isFile()) {
                    // realloc if necessary
                    while (buf.length < item.size) {
                        buf = Buffer.alloc(buf.length * 2);
                    }
                    // read the content and return a string
                    fs.readSync(fd, buf, 0, item.size, item.ino);
                    return buf.toString("utf-8", 0, item.size);
                }
                throw Error(`NO SUCH FILE ${filepath} in static file system.`);
            },
            realpathSync: (filepath) => {
                return filepath;
            },
            statSync: (filepath) => {
                return index[filepath] || {};
            }
        };
    });
}
exports.staticfs = staticfs;
//# sourceMappingURL=staticfs.js.map